@@ -336,7 +336,9 @@ def debhelper_argument_order(line, target):
 def override_dh_auto_test_drop_options(rule):
     if b'override_dh_auto_test' not in rule.targets:
         return
-    if rule.lines[1] != b'ifeq (,$(filter nocheck,$(DEB_BUILD_OPTIONS)))' or rule.lines[-1] != b'endif':
+    if (len(rule.lines) < 2
+            or rule.lines[1] != b'ifeq (,$(filter nocheck,$(DEB_BUILD_OPTIONS)))'
+            or rule.lines[-1] != b'endif'):
         return
     subitems.add(
         'Drop check for DEB_BUILD_OPTIONS containing "nocheck", '
