@@ -36,7 +36,6 @@ import sys
 from tempfile import TemporaryDirectory
 from typing import Optional, Tuple, List, Dict
 from urllib.parse import urlparse
-import warnings
 
 
 from debian.changelog import Changelog, Version, get_maintainer, format_date
@@ -76,6 +75,7 @@ from upstream_ontologist.debian import (
     valid_debian_package_name,
 )
 
+from breezy.plugins.debian.upstream import PackageVersionNotPresent
 from breezy.plugins.debian.upstream.pristinetar import get_pristine_tar_source
 from breezy.plugins.debian.upstream.branch import (
     upstream_version_add_revision,
@@ -109,6 +109,15 @@ from .publish import update_offical_vcs, NoVcsLocation, VcsAlreadySpecified
 from .standards_version import latest_standards_version
 
 
+class BuildSystemProcessError(Exception):
+    """Error processing buildsystem-specific part of debianization."""
+
+    def __init__(self, buildsystem, message, inner=None):
+        self.buildsystem = buildsystem
+        self.message = message
+        self.inner = inner
+
+
 class DebianizedPackageRequirementMismatch(Exception):
     """Debianized package does not satisfy requirement."""
 
@@ -141,7 +150,9 @@ class NoUpstreamReleases(Exception):
         self.name = name
 
 
-def write_changelog_template(path, source_name, version, wnpp_bugs=None):
+def write_changelog_template(path, source_name, version, author=None, wnpp_bugs=None):
+    if author is None:
+        author = get_maintainer()
     if wnpp_bugs:
         closes = " Closes: " + ", ".join([("#%d" % (bug,)) for bug, kind in wnpp_bugs])
     else:
@@ -153,7 +164,7 @@ def write_changelog_template(path, source_name, version, wnpp_bugs=None):
         distributions="UNRELEASED",
         urgency="low",
         changes=["", "  * Initial release." + closes, ""],
-        author="%s <%s>" % get_maintainer(),
+        author="%s <%s>" % author,
         date=format_date(),
     )
     with open(path, "w") as f:
@@ -164,38 +175,30 @@ async def find_archived_wnpp_bugs(source_name):
     try:
         from .udd import connect_udd_mirror
     except ModuleNotFoundError:
-        warnings.warn("asyncpg not available, unable to find wnpp bugs.")
+        logging.warning("asyncpg not available, unable to find wnpp bugs.")
         return []
-    conn = await connect_udd_mirror()
-    return [
-        (row[0], row[1])
-        for row in await conn.fetch(
-            """\
+    async with await connect_udd_mirror() as conn:
+        return [
+            (row[0], row[1])
+            for row in await conn.fetch("""\
 select id, substring(title, 0, 3) from archived_bugs where package = 'wnpp' and
 title like 'ITP: ' || $1 || ' -- %' OR
 title like 'RFP: ' || $1 || ' -- %'
-""",
-            source_name,
-        )
-    ]
+""", source_name)]
 
 
 async def find_wnpp_bugs(source_name):
     try:
         from .udd import connect_udd_mirror
     except ModuleNotFoundError:
-        warnings.warn("asyncpg not available, unable to find wnpp bugs.")
+        logging.warning("asyncpg not available, unable to find wnpp bugs.")
         return []
-    conn = await connect_udd_mirror()
-    return [
-        (row[0], row['type'])
-        for row in await conn.fetch(
-            """\
+    async with await connect_udd_mirror() as conn:
+        return [
+            (row[0], row['type'])
+            for row in await conn.fetch("""\
 select id, type from wnpp where source = $1 and type in ('ITP', 'RFP')
-""",
-            source_name,
-        )
-    ]
+""", source_name)]
 
 
 MINIMUM_CERTAINTY = "possible"  # For now..
@@ -273,7 +276,12 @@ def import_upstream_version_from_dist(
         wt, subpath, upstream_source, source_name, upstream_version,
         session):
     from breezy.plugins.debian import default_orig_dir
-    from breezy.plugins.debian.merge_upstream import get_tarballs, do_import
+    from breezy.plugins.debian.merge_upstream import (
+        get_tarballs,
+        do_import,
+        get_existing_imported_upstream_revids,
+        )
+    from breezy.plugins.debian.import_dsc import UpstreamAlreadyImported
 
     orig_dir = os.path.abspath(default_orig_dir)
 
@@ -293,11 +301,17 @@ def import_upstream_version_from_dist(
         upstream_revisions = upstream_source\
             .version_as_revisions(source_name, upstream_version)
         files_excluded = None
-        imported_revids = do_import(
-            wt, subpath, tarball_filenames, source_name, upstream_version,
-            current_version=None, upstream_branch=upstream_source.upstream_branch,
-            upstream_revisions=upstream_revisions,
-            merge_type=None, files_excluded=files_excluded)
+        try:
+            imported_revids = do_import(
+                wt, subpath, tarball_filenames, source_name, upstream_version,
+                current_version=None, upstream_branch=upstream_source.upstream_branch,
+                upstream_revisions=upstream_revisions,
+                merge_type=None, files_excluded=files_excluded)
+        except UpstreamAlreadyImported as e:
+            logging.warning(
+                'Upstream release %s already imported.', e.version)
+            imported_revids = get_existing_imported_upstream_revids(
+                upstream_source, source_name, upstream_version)
         pristine_revids = {}
         for (component, tag_name, revid, pristine_tar_imported) in imported_revids:
             pristine_revids[component] = revid
@@ -627,10 +641,14 @@ def process_cargo(es, session, wt, subpath, debian_path, upstream_version, metad
     control = es.enter_context(DebcargoControlShimEditor.from_debian_dir(wt.abspath(debian_path), upstream_name, upstream_version))
     # Only set semver_suffix if this is not the latest version
     import semver
-    desired_version = semver.VersionInfo.parse(upstream_version)
+    try:
+        desired_version = semver.VersionInfo.parse(upstream_version)
+    except ValueError as e:
+        raise BuildSystemProcessError(buildsystem, str(e), e)
     data = load_crate_info(upstream_name)
     if data is None:
-        raise Exception('Crate does not exist' % upstream_name)
+        raise BuildSystemProcessError(
+            buildsystem, 'Crate does not exist' % upstream_name)
     for version_info in data['versions']:
         available_version = semver.VersionInfo.parse(version_info['num'])
         if (available_version.major, available_version.minor) > (desired_version.major, desired_version.minor):
@@ -831,7 +849,8 @@ def debianize(  # noqa: C901
     upstream_version_kind: str = "auto",
     debian_revision: str = "1",
     upstream_version: Optional[str] = None,
-    requirement: Optional[Requirement] = None
+    requirement: Optional[Requirement] = None,
+    team: Optional[str] = None
 ):
     if committer is None:
         committer = get_committer(wt)
@@ -943,6 +962,12 @@ def debianize(  # noqa: C901
 
             source = control.source
 
+            if team:
+                control.source['Maintainer'] = team
+                uploader = '%s <%s>' % get_maintainer()
+                if uploader != team:
+                    control.source['Uploaders'] = uploader
+
             if not valid_debian_package_name(source['Source']):
                 raise SourcePackageNameInvalid(source['Source'])
 
@@ -958,6 +983,7 @@ def debianize(  # noqa: C901
                 wt.abspath(os.path.join(debian_path, "changelog")),
                 source["Source"],
                 version,
+                get_maintainer(),
                 wnpp_bugs,
             )
 
@@ -984,7 +1010,8 @@ def debianize(  # noqa: C901
 
     with wt.lock_write():
 
-        lintian_fixers = available_lintian_fixers(force_subprocess=force_subprocess)
+        lintian_fixers = available_lintian_fixers(
+            force_subprocess=force_subprocess)
 
         run_lintian_fixers(
             wt,
@@ -1003,7 +1030,8 @@ def debianize(  # noqa: C901
         )
 
         try:
-            result.vcs_url = update_offical_vcs(wt, subpath=subpath, committer=committer)
+            result.vcs_url = update_offical_vcs(
+                wt, subpath=subpath, committer=committer)
         except VcsAlreadySpecified:
             pass
         except NoVcsLocation:
@@ -1133,11 +1161,20 @@ def find_apt_upstream(requirement):
                     CargoCrateRequirement(name, version=version, features=features))
 
 
+def find_or_upstream(requirement) -> Optional[UpstreamInfo]:
+    for req in requirement.elements:
+        info = find_upstream(req)
+        if info is not None:
+            return info
+    return None
+
+
 UPSTREAM_FINDER = {
     'python-package': find_python_package_upstream,
     'go-package': find_go_package_upstream,
     'cargo-crate': find_cargo_crate_upstream,
     'apt': find_apt_upstream,
+    'or': find_or_upstream,
     }
 
 
@@ -1212,12 +1249,13 @@ class SimpleTrustedAptRepo(object):
             f.write(packages)
 
 
-def report_fatal(code, description, hint=None):
+def report_fatal(code, description, hint=None, details=None):
     if os.environ.get('SVP_API') == '1':
         with open(os.environ['SVP_RESULT'], 'w') as f:
             json.dump({
                 'result_code': code,
-                'description': description}, f)
+                'description': description,
+                'details': details}, f)
     logging.fatal('%s', description)
     if hint:
         logging.info('%s', hint)
@@ -1336,6 +1374,9 @@ def main(argv=None):  # noqa: C901
     parser.add_argument(
         '--dist-command', type=str,
         help='Dist command', default=os.environ.get('DIST'))
+    parser.add_argument(
+        '--team', type=str,
+        help='Maintainer team ("$NAME <$EMAIL>")')
     parser.add_argument('upstream', nargs='?', type=str)
 
     args = parser.parse_args(argv)
@@ -1401,6 +1442,15 @@ def main(argv=None):  # noqa: C901
                 create_dist=create_dist,
                 upstream_version=args.upstream_version,
             )
+        except PackageVersionNotPresent:
+            if args.upstream_version:
+                report_fatal(
+                    'requested-version-missing',
+                    'Requested version %s not present upstream' %
+                    args.upstream_version)
+            else:
+                # For now
+                raise
         except DistCommandFailed as e:
             report_fatal(e.kind or "dist-command-failed", e.error)
             return 1
@@ -1428,6 +1478,12 @@ def main(argv=None):  # noqa: C901
                 'no-build-tools',
                 "Unable to find any build systems in upstream sources.")
             return 1
+        except DetailedFailure as e:
+            report_fatal(
+                'debianize-' + e.error.kind,
+                str(e),
+                details=(e.error.json() if e.error else None))
+            return 1
         except OSError as e:
             if e.errno == errno.ENOSPC:
                 report_fatal('no-space-on-device', str(e))
@@ -1512,7 +1568,8 @@ def main(argv=None):  # noqa: C901
                     debian_revision=args.debian_revision,
                     upstream_version=upstream_info.version,
                     upstream_version_kind=args.upstream_version_kind,
-                    requirement=requirement)
+                    requirement=requirement,
+                    team=args.team)
                 do_build(
                     new_wt, new_subpath, self.apt_repo.directory,
                     extra_repositories=self.apt_repo.sources_lines())
